"""This package defines the sub-handlers and the main one."""

from web3data.handlers.address import AddressHandler
from web3data.handlers.api import APIHandler
from web3data.handlers.block import BlockHandler
from web3data.handlers.contract import ContractHandler
from web3data.handlers.market import MarketHandler
from web3data.handlers.signature import SignatureHandler
from web3data.handlers.token import TokenHandler
from web3data.handlers.transaction import TransactionHandler
from web3data.handlers.websocket import WebsocketHandler
